<?php
const UNKNOWN_PERSON = '_';

function getResizeHeight($width, $height, $resizeWidth)
{
    return intval($height * ($resizeWidth / $width));
}

function getPhotos($path): array
{
    $files = [];
    $dir = dir($path);
    while ($file = $dir->read()) {
        if (strpos($file, "jpg") !== false) {
            $files[] = $file;
        }
    }
    $dir->close();
    sort($files);
    return $files;
}

function isJointPhoto($photo, $people): bool
{
    $found = 0;
    foreach ($people as $person) {
        if (strpos($photo, $person) !== false) {
            $found++;
            if ($found >= 2) {
                return true;
            }
        }
    }
    return false;
}

function getPhotosByPerson($allPhotos, $people): array
{
    $photosByPerson = [];
    foreach ($allPhotos as $photo) {
        if (defined('SKIP_MULTI_PERSON_PHOTOS') && isJointPhoto($photo, $people)) continue;
        $isPersonFound = false;
        foreach ($people as $person) {
            if (strpos($photo, $person) !== false) {
                $photosByPerson[$person][] = $photo;
                $isPersonFound = true;
            }
        }
        if (!$isPersonFound) {
            $photosByPerson[UNKNOWN_PERSON][] = $photo;
        }
    }
    return $photosByPerson;
}

function getShrunkPhotos($path, $people, $minPhotosCount = 1): array
{
    $allPhotos = getPhotos($path);
    $photosByPerson = getPhotosByPerson($allPhotos, $people);
    $minSize = PHP_INT_MAX;
    foreach ($photosByPerson as $person => $arr) {
        $arrSize = sizeof($arr);
        echo $person . ': ' . $arrSize . PHP_EOL;
        if ($arrSize < $minPhotosCount) {
            echo 'Too small photos for ' . $person . ', exiting' . PHP_EOL;
            exit;
        }
        if ($arrSize < $minSize) {
            $minSize = $arrSize;
        }
    }
    echo 'minSize=' . $minSize . PHP_EOL;
    $result = [];
    foreach ($photosByPerson as $person => $arr) {
        echo $person . ': shrunk from ' . sizeof($arr) . ' to ' . $minSize . PHP_EOL;
        $shrunk = array_slice($arr, 0, $minSize);
        $result = array_merge($result, $shrunk);
    }
    return $result;
}

function getPaddedPhotos($path, $people): array
{
    $allPhotos = getPhotos($path);
    $photosByPerson = getPhotosByPerson($allPhotos, $people);
    $maxSize = 0;
    foreach ($photosByPerson as $person => $arr) {
        // if ($person == UNKNOWN_PERSON) continue;
        $arrSize = sizeof($arr);
        echo $person . ': ' . $arrSize . PHP_EOL;
        if ($arrSize > $maxSize) {
            $maxSize = $arrSize;
        }
    }
    $result = [];
    foreach ($photosByPerson as $person => $arr) {
        $arrSize = sizeof($arr);
        echo $person . ': pad from ' . $arrSize . ' to ' . $maxSize . PHP_EOL;
        for ($i = 0, $j = 0; $i < $maxSize; $i++, $j++) {
            $result[] = $arr[$j];
            if ($j == $arrSize - 1) {
                $j = 0;
            }
        }
    }
    return $result;
}

function getPaddedAndShrunkPhotosToUser($path, $people, $user)
{
    $allPhotos = getPhotos($path);
    $photosByPerson = getPhotosByPerson($allPhotos, $people);
    $countForUser = sizeof($photosByPerson[$user]);
    echo 'Needed count: ' . $countForUser . PHP_EOL;
    $result = [];
    foreach ($photosByPerson as $person => $arr) {
        $arrSize = sizeof($arr);
        if ($arrSize < $countForUser) {
            echo $person . ': pad from ' . $arrSize . ' to ' . $countForUser . PHP_EOL;
            for ($i = 0, $j = 0; $i < $countForUser; $i++, $j++) {
                $result[] = $arr[$j];
                if ($j == $arrSize - 1) {
                    $j = 0;
                }
            }
        } elseif ($arrSize > $countForUser) {
            echo $person . ': shrunk from ' . $arrSize . ' to ' . $countForUser . PHP_EOL;
            $shrunk = array_slice($arr, 0, $countForUser);
            $result = array_merge($result, $shrunk);
        } else {
            echo $person . ': use ' . $arrSize . PHP_EOL;
            $result = array_merge($result, $arr);
        }
    }
    return $result;
}


function getPaddedAndShrunkPhotosToCount($path, $people, $count)
{
    $allPhotos = getPhotos($path);
    $photosByPerson = getPhotosByPerson($allPhotos, $people);
    echo 'Needed count: ' . $count . PHP_EOL;
    $result = [];
    foreach ($photosByPerson as $person => $arr) {
        $arrSize = sizeof($arr);
        if ($arrSize < $count) {
            echo $person . ': pad from ' . $arrSize . ' to ' . $count . PHP_EOL;
            for ($i = 0, $j = 0; $i < $count; $i++, $j++) {
                $result[] = $arr[$j];
                if ($j == $arrSize - 1) {
                    $j = 0;
                }
            }
        } elseif ($arrSize > $count) {
            echo $person . ': shrunk from ' . $arrSize . ' to ' . $count . PHP_EOL;
            $shrunk = array_slice($arr, 0, $count);
            $result = array_merge($result, $shrunk);
        } else {
            echo $person . ': use ' . $arrSize . PHP_EOL;
            $result = array_merge($result, $arr);
        }
    }
    return $result;
}

function resizeImage($path, $x, $y, $width, $height, $resizeWidth)
{
    $im = imagecreatefromjpeg($path);
    if (!$im) {
        DebMes('Unable load image ' . $path);
        return false;
    }
    $resizeHeight = $height * ($resizeWidth / $width);
    $cropped = imagecrop($im, ['x' => $x, 'y' => $y, 'width' => $width, 'height' => $height]);
    $resized = imagecreatetruecolor($resizeWidth, $resizeHeight);
    imagecopyresampled($resized, $cropped, 0, 0, 0, 0, $resizeWidth, $resizeHeight, $width, $height);
//    header("Content-Type: image/jpeg");
//    imagejpeg($resized);
//    exit;
    imagedestroy($cropped);
    imagedestroy($im);
    return $resized;
}

function parseImage($path,
                    $x,
                    $y,
                    $width,
                    $height,
                    $resizeWidth): array
{
    $resized = resizeImage($path, $x, $y, $width, $height, $resizeWidth);
    $resizeHeight = getResizeHeight($width, $height, $resizeWidth);
    $result = [];
    for ($y = 0; $y < $resizeHeight; $y++) {
        for ($x = 0; $x < $resizeWidth; $x++) {
//            $rgb = imagecolorat($resized, $x, $y) / 16777215;
//            $result[] = $rgb;
            $rgb = imagecolorat($resized, $x, $y);
            $r = ($rgb >> 16) & 0xFF;
            $g = ($rgb >> 8) & 0xFF;
            $b = $rgb & 0xFF;
            $gray = ($r + $g + $b) / 3;
            $result[] = $gray / 255;
        }
    }
    imagedestroy($resized);
    return $result;
}

function generateXorData(
    $xorFile,
    $people,
    $photosDir,
    $images,
    $x,
    $y,
    $width,
    $height,
    $resizeWidth
)
{
    $resizeHeight = getResizeHeight($width, $height, $resizeWidth);
    $imagesCount = sizeof($images);

    file_put_contents($xorFile, $imagesCount . ' ' . ($resizeWidth * $resizeHeight) . ' ' . sizeof($people) . "\n");

    foreach ($images as $file) {
        $pixels = parseImage($photosDir . "/" . $file, $x, $y, $width, $height, $resizeWidth);
        file_put_contents($xorFile, implode(' ', $pixels) . "\n", FILE_APPEND);

        $expected = [];
        foreach ($people as $person) {
            $expected[] = intval(strpos($file, $person) !== false);
        }
        file_put_contents($xorFile, implode(' ', $expected) . "\n", FILE_APPEND);
    }
}
