#!/usr/bin/php -d memory_limit=2048M
<?php
ini_set('max_execution_time', '1000000');

$id = $argv[1];
if (empty($id)) exit("Source ID not passed");

require(__DIR__ . '/functions.php');
include_once(__DIR__ . "/../../config.php");
include_once(__DIR__ . "/../../lib/loader.php");
include_once(__DIR__ . "/../../load_settings.php");

function deleteDir($src): void
{
    if (file_exists($src)) {
        $dir = opendir($src);
        while (false !== ($file = readdir($dir))) {
            if (($file != '.') && ($file != '..')) {
                if (is_dir($src . '/' . $file)) {
                    deleteDir($src . '/' . $file);
                } else {
                    unlink($src . '/' . $file);
                }
            }
        }
        closedir($dir);
        rmdir($src);
    }
}

$startTime = time();
$source = SQLSelectOne("SELECT * FROM neuroconcierge_sources WHERE ID=" . $id);
$peopleRaw = SQLSelect("SELECT * FROM neuroconcierge_people WHERE source=" . $id . " ORDER BY NAME");

$people = array();
foreach ($peopleRaw as $item) {
    $people[] = $item['NAME'];
}

file_put_contents(__DIR__ . "/data/people$id.json", json_encode($people));
echo 'People: ' . implode(', ', $people) . PHP_EOL;
echo 'Save people list' . PHP_EOL;

if (!$source['USE_MULTI_PERSON_PHOTOS']) {
    define('SKIP_MULTI_PERSON_PHOTOS', 1);
}

$photosDir = __DIR__ . '/data/photos/' . $id;
switch ($source['SHRINK_DATA']) {
    case 0:
        $files = getPhotos($photosDir);
        break;
    case 1:
        $files = getShrunkPhotos($photosDir, $people);
        break;
    case 2:
        $files = getPaddedPhotos($photosDir, $people);
        break;
    case 3:
        $files = getPaddedAndShrunkPhotosToCount($photosDir, $people, $source['SHRINK_DATA_VALUE']);
        break;
    case 4:
        if (!in_array($source['SHRINK_DATA_PERSON'], $people)) exit("Cannot find person with name '" . $source['SHRINK_DATA_PERSON'] . "'");
        $files = getPaddedAndShrunkPhotosToUser($photosDir, $people, $source['SHRINK_DATA_PERSON']);
        break;
    default:
        exit("Unknown shrink method");
}

$stopFilePath = __DIR__ . '/data/stop' . $id . '.pid';
if (file_exists($stopFilePath)) {
    unlink($stopFilePath);
}

echo 'Write to XOR file ' . sizeof($files) . ' records' . PHP_EOL;
$xorData = __DIR__ . '/data/data' . $id . '.xor';
$trainData = __DIR__ . '/data/train' . $id . '.net';
generateXorData(
    $xorData,
    $people,
    $photosDir,
    $files,
    $source['X'],
    $source['Y'],
    $source['WIDTH'],
    $source['HEIGHT'],
    $source['RESIZE_WIDTH']);
echo 'XOR data successfully saved to ' . $xorData . PHP_EOL . PHP_EOL;
echo 'Initialize training' . PHP_EOL;

$inputNeuronsCount = $source['RESIZE_WIDTH'] * getResizeHeight($source['WIDTH'], $source['HEIGHT'], $source['RESIZE_WIDTH']);
echo "inputNeuronsCount=$inputNeuronsCount" . PHP_EOL;
$outputNeuronsCount = sizeof($people);
echo "outputNeuronsCount=$outputNeuronsCount" . PHP_EOL;
$layersCount = $source['HIDDEN_LAYERS_COUNT'] + 2;
echo "layersCount=$layersCount" . PHP_EOL;
$firstHiddenLayerNeuronsCount = $source['FIRST_HIDDEN_LAYER_NEURONS'];
echo "firstHiddenLayerNeuronsCount=$firstHiddenLayerNeuronsCount" . PHP_EOL;
$secondHiddenLayerNeuronsCount = $source['SECOND_HIDDEN_LAYER_NEURONS'];
echo "secondHiddenLayerNeuronsCount=$secondHiddenLayerNeuronsCount" . PHP_EOL;
$divisor = pow(10, $source['ACCURACY']);
echo "divisor=$divisor" . PHP_EOL;
$desiredError = 1 / $divisor;
echo "desiredError=$desiredError" . PHP_EOL;
$maxEpochs = 100000;
$currentEpoch = 0;
$epochsBetweenSnapshots = 0;
$epochsSinceLastSnapshot = 0;
$maxSnapshots = 10;
$snapshots = array();
$hiddenActivation = FANN_SIGMOID_STEPWISE;
$outputActivation = FANN_SIGMOID_STEPWISE;
$trainingAlgorithm = FANN_TRAIN_QUICKPROP;

$dirName = __DIR__ . '/data/snapshots' . $id;

if (!is_file($dirName)) {
    mkdir($dirName);
}

$pseudoMseResult = $desiredError * $divisor;
$best_mse = $pseudoMseResult;

$ann_name = "xor_$inputNeuronsCount($firstHiddenLayerNeuronsCount)$outputNeuronsCount - ($hiddenActivation)($outputActivation)$trainingAlgorithm" . '_' . mt_rand(100, 999);

$log_file = $dirName . '/training-log_' . $ann_name . '.csv';
$log = fopen($log_file, 'w');

$ann = fann_create_standard($layersCount, $inputNeuronsCount, $firstHiddenLayerNeuronsCount, $secondHiddenLayerNeuronsCount, $outputNeuronsCount);

if ($ann) {
    echo 'Training... ' . PHP_EOL;

    fann_set_activation_function_hidden($ann, $hiddenActivation);
    fann_set_activation_function_output($ann, $outputActivation);
    fann_set_training_algorithm($ann, $trainingAlgorithm);

    $train_data = fann_read_train_from_file($xorData);
    if ($source['SHUFFLE_DATA']) {
        fann_shuffle_train_data($train_data);
    }

    $isStopFileExists = false;
    while ($pseudoMseResult > $desiredError && $currentEpoch <= $maxEpochs && !$isStopFileExists) {
        $currentEpoch++;
        $epochsSinceLastSnapshot++;

        $isStopFileExists = file_exists($stopFilePath);

        $pseudoMseResult = fann_train_epoch($ann, $train_data);
        echo 'Epoch ' . $currentEpoch . ': ' . $pseudoMseResult . PHP_EOL;

        $logData = $currentEpoch . ',' . $pseudoMseResult;

        if (($epochsSinceLastSnapshot >= $epochsBetweenSnapshots) && ($pseudoMseResult < $best_mse)) {
            $best_mse = $pseudoMseResult;
            fann_save($ann, "$dirName/$ann_name - $currentEpoch.net");
            echo 'Snapshot Taken' . PHP_EOL;
            $epochsSinceLastSnapshot = 0;
            $logData .= ',1';
            array_unshift($snapshots, "$dirName/$ann_name - $currentEpoch.net");
            $snapshotsCount = count($snapshots) - 1;

            if ($snapshotsCount > $maxSnapshots) {
                do {
                    unlink($snapshots[$snapshotsCount]);
                    unset($snapshots[$snapshotsCount]);
                    --$snapshotsCount;
                } while ($snapshotsCount > $maxSnapshots);
                $snapshots = array_values(array_filter($snapshots));
            }
        } else {
            $logData .= ',0';
        }
        fwrite($log, $logData . PHP_EOL);
    }
    fclose($log);

    if ($isStopFileExists) {
        echo 'Training Stopped! Saving current result to ' . $trainData . PHP_EOL;
    } else {
        echo 'Training Complete! Saving final result to ' . $trainData . PHP_EOL;
    }
    fann_save($ann, $trainData);
    fann_destroy($ann);
    echo 'Remove temporary directory' . PHP_EOL;
    deleteDir($dirName);
}
$time = time() - $startTime;
$seconds = $time % 60;
$time = ($time - $seconds) / 60;
$minutes = $time % 60;
$hours = ($time - $minutes) / 60;
echo 'Time spent: ' . sprintf('%02d:%02d:%02d', $hours, $minutes, $seconds) . PHP_EOL . PHP_EOL;
echo 'All Done!' . PHP_EOL;